using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;

namespace SampleClient
{
    public enum RobotType {ROBOT_DISCPUBLISHER = 0, ROBOT_DISCPUBLISHERII,ROBOT_DISCPUBLISHERPRO, ROBOT_UNK, ROBOT_RACKMOUNT_DPII, ROBOT_DISCPUBLISHER_XRP, ROBOT_DISCPUBLISHER_SE, ROBOT_DISCPUBLISHERPRO_XI, ROBOT_DISCPUBLISHER_4100, ROBOT_DISCPUBLISHER_4100_XRP};

    public partial class Form1 : Form
    {
        private SystemStatus mySystemStatus;
        private StatusFile myStatusFile;
        private string server, statusFile, logFile;
        private Robot currentRobot;

        public Form1()
        {
            InitializeComponent();
            this.serviceController1.MachineName = Environment.MachineName;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.StartupVersion == "PTRobot")
            {
                this.tabControl1.TabPages.Clear();
                this.tabControl1.TabPages.Add(this.tabPagePTRobot);
                this.panel2.Visible = false;
                this.timer1.Stop();
                if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Stopped &&
                    this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StopPending)
                    this.serviceController1.Stop();
            }
            else
            {
                this.tabControl1.TabPages.Clear();
                this.tabControl1.TabPages.Add(this.tabPageJobs);
                this.tabControl1.TabPages.Add(this.tabPageStatus);
                this.tabControl1.TabPages.Add(this.tabPageTesting);
                this.panel2.Visible = true;
                this.timer1.Start();
                if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Running &&
                    this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StartPending)
                {
                    try
                    {
                        this.serviceController1.Start();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Unable to start service.  Try starting manually or run app as admin");
                    }
                }
            }

            UpdateServiceButton();


            this.comboBoxMsg.Items.Add("USB Disconnect");
            this.comboBoxMsg.Items.Add("USB Connect");
            this.comboBoxMsg.Items.Add("Check Bins");
            this.comboBoxMsg.Items.Add("Create Report");
            this.comboBoxMsg.Items.Add("Run Utility");
            this.comboBoxMsg.Items.Add("Align Printer");
            this.comboBoxMsg.SelectedIndex = 0;
            if (String.IsNullOrEmpty(Properties.Settings.Default.PTBurnDocPath))
                this.buttonPTBurnDoc.Enabled = false;
            else
                this.buttonPTBurnDoc.Enabled = true;

            if (String.IsNullOrEmpty(Properties.Settings.Default.PTBurnHistoryPath))
                this.buttonPTBurnHistory.Enabled = false;
            else
                this.buttonPTBurnHistory.Enabled = true;

            if (String.IsNullOrEmpty(Properties.Settings.Default.PTRobotDocPath))
                this.buttonPTRobotDoc.Enabled = false;
            else
                this.buttonPTRobotDoc.Enabled = true;

            if (String.IsNullOrEmpty(Properties.Settings.Default.PTRobotHistoryPath))
                this.buttonPTRobotHistory.Enabled = false;
            else
                this.buttonPTRobotHistory.Enabled = true;

            this.textBoxServer.Text = server = Environment.MachineName;

            RefreshRobotList();
        }
        private void SetRobot()
        {
            if (mySystemStatus.Robots.Count > 0)
            {
                currentRobot = mySystemStatus.Robots[this.comboBoxRobots.SelectedIndex] as Robot;

                this.linkLabelLogFile.Text = "\\\\" + this.server + "\\PTBurnJobs\\Log\\PTLog.txt";
                this.linkLabelStatusFile.Text = "\\\\" + this.server + "\\PTBurnJobs\\Status\\" + currentRobot.StatusFile;
                this.linkLabelSystemStatusFile.Text = "\\\\" + this.server + "\\PTBurnJobs\\Status\\SystemStatus.txt";

                this.myStatusFile = new StatusFile(this.linkLabelStatusFile.Text);

                string sFile;
                if (currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHERII)
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPII.png";
                else if (currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_SE)
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPSE.png";
                else if ((currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_XRP) ||
                          (currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_4100_XRP ))
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPXRP.png";
                else if ((currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHERPRO) ||
                            (currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHERPRO_XI) ||
                            (currentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_4100))
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPPRO.png";
                else if (currentRobot.RobotType == RobotType.ROBOT_RACKMOUNT_DPII)
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPXR.png";
                else
                    sFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\NoneFound.png";

                if (File.Exists(sFile))
                {
                    Image robotImage = Image.FromFile(sFile);

                    this.pictureBoxRobtMain.Image = robotImage;
                }

                this.ptBurnJobsCtrl1.SetRobot(currentRobot, this.server);
                this.ptBurnStatusCtrl1.SetRobot(currentRobot, this.server);
                this.ptBurnTestingCtrl1.SetRobot(currentRobot, this.server);
                this.buttonSendMsg.Enabled = true;
            }
            else
            {
                currentRobot = null;
                this.linkLabelLogFile.Text = "";
                this.linkLabelStatusFile.Text = "";
                this.myStatusFile = null;
                this.pictureBoxRobtMain.Image = null;
                this.buttonSendMsg.Enabled = false;

                this.ptBurnJobsCtrl1.SetRobot(null, this.server);
                this.ptBurnStatusCtrl1.SetRobot(null, this.server);
                this.ptBurnTestingCtrl1.SetRobot(null, this.server);
            }
        }
   


        private void linkLabelStatusFile_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (File.Exists(linkLabelStatusFile.Text))
                Process.Start(linkLabelStatusFile.Text);
        }

        private void linkLabelLogFile_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (File.Exists(linkLabelLogFile.Text))
                Process.Start(linkLabelLogFile.Text);
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.SelectedIndex == 3)
                this.panel2.Visible = false;
            else
                this.panel2.Visible = true;
        }

        private void UpdateServiceButton()
        {
            this.serviceController1.Refresh();
            if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Running &&
                this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StartPending)
                this.buttonService.Text = "Service Stopped\n(Click to Start)";
            else
                this.buttonService.Text = "Service Running\n(Click to Stop)";
        }

        private void buttonSendMsg_Click(object sender, EventArgs e)
        {
            string sJobName = "Message";
            string sJobFile;
            int nIndex = 1;
            while (true)
            {
                sJobFile = "\\\\" + this.textBoxServer.Text + "\\PTBurnJobs\\" + sJobName + ".ptm";
                if (File.Exists(sJobFile))
                {
                    sJobName = "Message (" + nIndex.ToString() + ")";
                    nIndex++;
                }
                else
                {
                    break;
                }
            }
            
            TextWriter tw;
            if( Properties.Settings.Default.UnicodeEncoding)
                tw = new StreamWriter(sJobFile, false, Encoding.Unicode);
            else
                tw = new StreamWriter(sJobFile);

            if (tw != null)
            {
                if (comboBoxMsg.SelectedIndex == 0)
                {
                    tw.WriteLine("Message=USB_DISCONNECT");
                    tw.WriteLine("RobotName=" + this.comboBoxRobots.SelectedItem.ToString());
                }
                else if (comboBoxMsg.SelectedIndex == 1)
                {
                    tw.WriteLine("Message=USB_CONNECT");
                    //tw.WriteLine("RobotName=" + this.comboBox1.SelectedItem.ToString());
                }
                else if (comboBoxMsg.SelectedIndex == 2)
                {
                    tw.WriteLine("Message=CHECK_DISCSINBIN");
                    tw.WriteLine("ClientID=Administrator");
                    tw.WriteLine("RobotName=" + this.comboBoxRobots.SelectedItem.ToString());
                }
                else if (comboBoxMsg.SelectedIndex == 3)
                {
                    tw.WriteLine("Message=CREATE_REPORT");
                }
                else if (comboBoxMsg.SelectedIndex == 4)
                {
                    File.Copy(Environment.CurrentDirectory + "\\HelloWorld.exe", "\\\\" + this.textBoxServer.Text + "\\PTBurnJobs\\HelloWorld.exe",true);
                    tw.WriteLine("Message=RUN_UTILITY");
                    tw.WriteLine("PathFrom=HelloWorld.exe");
                }
                else if (comboBoxMsg.SelectedIndex == 5)
                {
                    tw.WriteLine("Message=ALIGN_PRINTER");
                    tw.WriteLine("ClientID=Administrator");
                    tw.WriteLine("RobotName=" + this.comboBoxRobots.SelectedItem.ToString());
                }

                tw.Close();
            }
        }

        private void buttonRefresh_Click(object sender, EventArgs e)
        {
            RefreshRobotList();
        }
        private void RefreshRobotList()
        {
            int nRetries = 5;
            mySystemStatus = new SystemStatus("\\\\" + this.server + "\\PTBurnJobs\\Status\\");
            comboBoxRobots.Items.Clear();
            while (true)
            {
                mySystemStatus.Refresh();


                foreach (Robot myRobot in mySystemStatus.Robots)
                    comboBoxRobots.Items.Add(myRobot.Name);

                if (comboBoxRobots.Items.Count > 0)
                {
                    this.comboBoxRobots.Focus();
                    this.comboBoxRobots.SelectedIndex = 0;
                    break;
                }
                else
                {
                    if (nRetries-- <= 0)
                        break;
                }
                System.Threading.Thread.Sleep(500);
            }
            SetRobot();
        }

        private void comboBoxRobots_SelectedIndexChanged(object sender, EventArgs e)
        {
            SetRobot();
        }

        private void UpdateRobotInfo()
        {
            if (currentRobot != null)
            {
                currentRobot.Refresh();
                if (currentRobot.DiscsInBin1 != 9999)
                {
                    if (currentRobot.DiscsInBin0 == 255)
                        this.labelBins.Text = "Left bin: ? Discs\n";
                    else
                        this.labelBins.Text = "Left bin: " + currentRobot.DiscsInBin0.ToString() + " Discs\n";

                    if (currentRobot.DiscsInBin1 == 255)
                        this.labelBins.Text += "Right bin: ? Discs";
                    else
                        this.labelBins.Text += "Right bin: " + currentRobot.DiscsInBin1.ToString() + " Discs\n";
                }
                else
                {
                    if (currentRobot.DiscsInBin0 == 255)
                        this.labelBins.Text = "Input bin: " + "? Discs";
                    else
                        this.labelBins.Text = "Input bin: " + currentRobot.DiscsInBin0.ToString() + " Discs";
                }
                this.labelCartridges.Text = "";
                if (currentRobot.CartType0 == CartridgeType.COLOR_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Color: " + currentRobot.CartFill0.ToString() + " %\n";
                }
                else if (currentRobot.CartType0 == CartridgeType.BLACK_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Black: " + currentRobot.CartFill0.ToString() + " %\n";
                }
                else
                    this.labelCartridges.Visible = false;

                if (currentRobot.CartType1 == CartridgeType.COLOR_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Color: " + currentRobot.CartFill1.ToString() + " %\n";
                }
                else if (currentRobot.CartType1 == CartridgeType.BLACK_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Black: " + currentRobot.CartFill1.ToString() + " %\n";
                }
                else if (currentRobot.CartType1 == CartridgeType.YELLOW_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Yellow: " + currentRobot.CartFill1.ToString() + " %\n";
                }

                
                // NOTE: for 4-color individual tanks this is assuming a certain cartridge order
                // Probably should improve this code in case the order is not Y,C,M
                if (currentRobot.CartType2 == CartridgeType.CYAN_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Cyan: " + currentRobot.CartFill2.ToString() + " %\n";
                }
                if (currentRobot.CartType3 == CartridgeType.MAGENTA_CARTRIDGE)
                {
                    this.labelCartridges.Text += "Magenta: " + currentRobot.CartFill3.ToString() + " %\n";
                }

            }
        }

        private void labelTitle_Click(object sender, EventArgs e)
        {

        }

        private void buttonPTBurnSample_Click(object sender, EventArgs e)
        {
            this.panel2.Visible = true;
            if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Running &&
                this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StartPending)
                this.serviceController1.Start();

            UpdateServiceButton();

            this.timer1.Start();

            Properties.Settings.Default.StartupVersion = "PTBurn";
            Properties.Settings.Default.Save();
            this.tabControl1.TabPages.Clear();
            this.tabControl1.TabPages.Add(this.tabPageJobs);
            this.tabControl1.TabPages.Add(this.tabPageStatus);
            this.tabControl1.TabPages.Add(this.tabPageTesting);
        }

        private void buttonPTBurnDoc_Click(object sender, EventArgs e)
        {
            Process.Start(Properties.Settings.Default.PTBurnDocPath);
        }

        private void buttonPTRobotSample_Click(object sender, EventArgs e)
        {
            if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Stopped &&
                this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StopPending)
                this.serviceController1.Stop();

            this.timer1.Stop();
            UpdateServiceButton();
            Properties.Settings.Default.StartupVersion = "PTRobot";
            Properties.Settings.Default.Save();
            this.tabControl1.TabPages.Clear();
            this.tabControl1.TabPages.Add(this.tabPagePTRobot);
            this.panel2.Visible = false;
        }

        private void buttonPTRobotDoc_Click(object sender, EventArgs e)
        {
            Process.Start(Properties.Settings.Default.PTRobotDocPath);
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.primera.com/developer/developer.html");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            this.UpdateRobotInfo();
        }

        private void buttonPTRobotHistory_Click(object sender, EventArgs e)
        {
            Process.Start(Properties.Settings.Default.PTRobotHistoryPath);
        }

        private void buttonPTBurnHistory_Click(object sender, EventArgs e)
        {
            Process.Start(Properties.Settings.Default.PTBurnHistoryPath);
        }

        private void buttonService_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.Running &&
                    this.serviceController1.Status != System.ServiceProcess.ServiceControllerStatus.StartPending)
                    this.serviceController1.Start();
                else
                    this.serviceController1.Stop();
            }
            catch
            {
            }

            UpdateServiceButton();
        }

        private void linkLabelSystemStatusFile_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (File.Exists(linkLabelSystemStatusFile.Text))
                Process.Start(linkLabelSystemStatusFile.Text);
        }

    }
}
