using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.IO;

namespace SampleClient
{

    
    public partial class FormRunTest : Form
    {
        public ArrayList JobsToRun;
        public UInt32 nIterations;
        public string sJobDir;
        public string sServer;
        public string sReportDir;
        private string sReportFile;
        public string sStatusFile;
        public string sLogFile;
        public bool RunIndependently;
        private StatusFile myStatusFile;
        private UInt32 nTotalJobs;
        private UInt32 nJobsCompleted;
        public UInt32 nAbortRate;
        public bool fRandomizeList;
        private string sResultsDirectory;


        public FormRunTest()
        {
            InitializeComponent();
            JobsToRun = new ArrayList();


        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            JobResult myResult = new JobResult();
            TextWriter tw;
            if( Properties.Settings.Default.UnicodeEncoding)
                tw = new StreamWriter(sReportFile, false, Encoding.Unicode);
            else
                tw = new StreamWriter(sReportFile);

            if (tw != null)
            {
                tw.WriteLine("Test Report generated " + DateTime.Now.ToString());
                tw.WriteLine("------------------------------------------");
                tw.Flush();
                //TODO Add Versions
                DirectoryInfo myJobsDir = new DirectoryInfo(sJobDir);
                FileInfo[] myFiles = myJobsDir.GetFiles();
                tw.WriteLine("Number of Jobs: " + JobsToRun.Count.ToString());

                for (int i = 0; i < nIterations; i++)
                {
                    if (backgroundWorker1.CancellationPending)
                    {
                        tw.Close();
                        return;
                    }
                    tw.WriteLine("Starting Iteration " + i.ToString());
                    foreach (string myJobName in JobsToRun)
                    {
                        if (backgroundWorker1.CancellationPending)
                        {
                            tw.Close();
                            return;
                        }
                        myResult.fStarting = true;
                        myResult.sStatus = "Running";
                        myResult.sJob = myJobName;
                        backgroundWorker1.ReportProgress(0, myResult);
                        tw.WriteLine("   Starting Job " + myJobName);
                        File.Copy(sJobDir +"\\"+ myJobName, "\\\\" + this.sServer + "\\PTBurnJobs\\" + myJobName);
                        bool fCompletedFound = false;
                        DateTime StartTime = DateTime.Now;
                        Int32 nAbortTiming = GetAbortTiming();
                        while (true)
                        {
                            if (backgroundWorker1.CancellationPending)
                            {
                                tw.Close();
                                return;
                            }
                            if (nAbortTiming > 0)
                            {
                                TimeSpan ts = DateTime.Now.Subtract(StartTime);
                                if (ts.Seconds > nAbortTiming)
                                {
                                    tw.WriteLine("   Aborting " + myJobName);
                                    tw.WriteLine("      Random Time to Abort " + nAbortTiming.ToString() + " seconds");
                                    tw.WriteLine("      Current Time " + DateTime.Now.ToString() + "  Start Time " + StartTime.ToString());
                                }
                            }
                            myStatusFile.RefreshJobs();
                            foreach (Job myJob in myStatusFile.Jobs)
                            {

                                if (myJob.CompletedJob && (myJobName.Contains(myJob.Name)))
                                {
                                    myResult.fStarting = false;
                                    myResult.sJob = myJob.Name;
                                    myResult.nBadDiscs = myJob.BadDiscs;
                                    myResult.nGoodDiscs = myJob.GoodDiscs;

                                    if (myJob.JobState == JobState.JOB_COMPLETED_SUCCESSFULLY)
                                    {
                                        tw.WriteLine("   " + myJob.Name + " Completed");
                                        myResult.sStatus = "Job Completed";
                                        myResult.fPassed = true;
                                     
                                    }
                                    else if (myJob.JobState == JobState.JOB_FAILED)
                                    {
                                        tw.WriteLine("   " + myJob.Name + " Failed");
                                        myResult.sStatus = "Job Failed";
                                        myResult.fPassed = false;
                                    }
                                    tw.WriteLine("      Good Discs: " + myJob.GoodDiscs);
                                    tw.WriteLine("      Bad Discs: " + myJob.BadDiscs);
                                    fCompletedFound = true;
                                    this.nJobsCompleted++;
                                    tw.Flush();
                                    backgroundWorker1.ReportProgress(0, myResult);
                                    break;

                                }
                            }
                            if (fCompletedFound)
                                break;
                            System.Threading.Thread.Sleep(1000);
                        }
                    }
                    File.Copy(Path.GetDirectoryName(sLogFile) + "\\PTBS_RobotLogD.dbg", sResultsDirectory + "\\PTBS_RobotLogD_" + i.ToString() + ".dbg");
                    File.Copy(Path.GetDirectoryName(sLogFile) + "\\PTBS_RobotLogT.dbg", sResultsDirectory + "\\PTBS_RobotLogT_" + i.ToString() + ".dbg");
                    File.Copy(sStatusFile, sResultsDirectory + "\\Status_" +i.ToString() +".txt");
                    File.Copy(sLogFile, sResultsDirectory + "\\Log_" + i.ToString() + ".txt"); 
                    File.Delete(sStatusFile);
                    File.Delete(sLogFile);
                }
                tw.Flush();
                tw.Close();
            }
        }

        private Int32 GetAbortTiming()
        {
            Random random = new Random();
            if (nAbortRate > 0)
            {
                int nRand = random.Next(1, 100);
                if (nRand < this.nAbortRate)
                    return nRand;
                else
                    return 0;
            }
            else
                return 0;
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            JobResult myResult = e.UserState as JobResult;
            UInt32 nTemp=0;



            for(int i=0; i<this.listView1.Items.Count; i++)
            {
                if(this.listView1.Items[i].SubItems[0].Text.Contains(myResult.sJob))
                {
                    this.listView1.Items[i].SubItems[1].Text = myResult.sStatus;
                    if (!myResult.fStarting)
                    {
                        int nPercent = (int)((this.nJobsCompleted / this.nTotalJobs) * 100);
                        this.progressBar1.Value = nPercent;
                        
                        if (myResult.fPassed)
                        {
                            nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[2].Text) + 1;
                            this.listView1.Items[i].SubItems[2].Text = nTemp.ToString();
                        }
                        else
                        {
                            nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[3].Text) + 1;
                            this.listView1.Items[i].SubItems[3].Text = nTemp.ToString();
                        }
                        nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[4].Text) + myResult.nGoodDiscs;
                        this.listView1.Items[i].SubItems[4].Text = nTemp.ToString();
                        nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[5].Text) + myResult.nBadDiscs;
                        this.listView1.Items[i].SubItems[5].Text = nTemp.ToString();
                    }
                }

            }
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            MessageBox.Show("Testing Complete.  See Complete Report for details.\n\n" + this.sReportFile);
        }

        private void FormRunTest_Load(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(this.sStatusFile))
                File.Delete(this.sStatusFile);

            if (!String.IsNullOrEmpty(this.sLogFile))
                File.Delete(this.sLogFile);

            foreach (string sJob in JobsToRun)
            {
                string[] sItem = { sJob, "Waiting to Run", "0", "0", "0", "0" };
                ListViewItem myItem = new ListViewItem(sItem);
                this.listView1.Items.Add(myItem);
            }
            nTotalJobs = this.nIterations * (uint)this.listView1.Items.Count;
            nJobsCompleted = 0;

            sResultsDirectory = sReportDir + "\\" + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Year.ToString() + "_" + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + DateTime.Now.Second.ToString();
            Directory.CreateDirectory(sResultsDirectory);
            sReportFile = sResultsDirectory + "\\Report.txt";
            System.Threading.Thread.Sleep(1000);
            myStatusFile = new StatusFile(sStatusFile);
            myStatusFile.Refresh();
            if (this.RunIndependently)
                this.backgroundWorker1.RunWorkerAsync();
            else
                this.backgroundWorkerRunAll.RunWorkerAsync();

        }

        private void ShuffleArrayList(ref ArrayList source)
        {

            ArrayList target = new ArrayList(source.Count);

            Random rnd = new Random();

            while (source.Count > 0)
            {

                int idx = rnd.Next(source.Count);

                target.Add(source[idx]);

                source.RemoveAt(idx);

            }

            source = target;

        }


        private void buttonCancel_Click(object sender, EventArgs e)
        {
            backgroundWorker1.CancelAsync();
            this.Close();
        }

        private void backgroundWorkerRunAll_DoWork(object sender, DoWorkEventArgs e)
        {
            JobResult myResult = new JobResult();
            
            TextWriter tw;
            if( Properties.Settings.Default.UnicodeEncoding)
                tw = new StreamWriter(sReportFile, false, Encoding.Unicode);
            else
                tw = new StreamWriter(sReportFile);

            if (tw == null)
                return;
           
            tw.WriteLine("Test Report generated " + DateTime.Now.ToString());
            tw.WriteLine("------------------------------------------");
            tw.Flush();
            //TODO Add Versions
            DirectoryInfo myJobsDir = new DirectoryInfo(sJobDir);
            FileInfo[] myFiles = myJobsDir.GetFiles();
            tw.WriteLine("Number of Jobs: " + JobsToRun.Count.ToString());

            for (int i = 0; i < nIterations; i++)
            {
                if (backgroundWorkerRunAll.CancellationPending)
                {
                    tw.Close();
                    return;
                }
                tw.WriteLine("Starting Iteration " + i.ToString());


                foreach (string myJobName in JobsToRun)
                {
                    if (backgroundWorkerRunAll.CancellationPending)
                    {
                        tw.Close();
                        return;
                    }


                    tw.WriteLine("   Starting Job " + myJobName);
                    File.Copy(sJobDir + "\\" + myJobName, "\\\\" + this.sServer + "\\PTBurnJobs\\" + myJobName);
                }
                bool fCompletedFound = false;
                DateTime StartTime = DateTime.Now;
                Int32 nAbortTiming = GetAbortTiming();
                while (true)
                {
                    if (backgroundWorkerRunAll.CancellationPending)
                    {
                        tw.Close();
                        return;
                    }
                    myStatusFile.RefreshJobs();
                    foreach (Job myJob in myStatusFile.Jobs)
                    {

                        if (myJob.CompletedJob)
                        {
                            myResult.fStarting = false;
                            myResult.sJob = myJob.Name;
                            myResult.nBadDiscs = myJob.BadDiscs;
                            myResult.nGoodDiscs = myJob.GoodDiscs;

                            if (myJob.JobState == JobState.JOB_COMPLETED_SUCCESSFULLY)
                            {
                                tw.WriteLine("   " + myJob.Name + " Completed");
                                myResult.sStatus = "Job Completed";
                                myResult.fPassed = true;

                            }
                            else if (myJob.JobState == JobState.JOB_FAILED)
                            {
                                tw.WriteLine("   " + myJob.Name + " Failed");
                                myResult.sStatus = "Job Failed";
                                myResult.fPassed = false;
                            }
                            tw.WriteLine("      Good Discs: " + myJob.GoodDiscs);
                            tw.WriteLine("      Bad Discs: " + myJob.BadDiscs);
                            fCompletedFound = true;
                            this.nJobsCompleted++;
                            tw.Flush();
                            backgroundWorkerRunAll.ReportProgress(0, myResult);
                            break;

                        }
                    }
                    if (this.nJobsCompleted == JobsToRun.Count)
                    {
                        tw.WriteLine("All Jobs Completed");
                        break;
                    }
                    System.Threading.Thread.Sleep(1000);
                }
                File.Copy(Path.GetDirectoryName(sLogFile) + "\\PTBS_RobotLogD.dbg", sResultsDirectory + "\\PTBS_RobotLogD_" + i.ToString() + ".dbg");
                File.Copy(Path.GetDirectoryName(sLogFile) + "\\PTBS_RobotLogT.dbg", sResultsDirectory + "\\PTBS_RobotLogT_" + i.ToString() + ".dbg");
                File.Copy(sStatusFile, sResultsDirectory + "\\Status_" + i.ToString() + ".txt");
                File.Copy(sLogFile, sResultsDirectory + "\\Log_" + i.ToString() + ".txt");
                File.Delete(sStatusFile);
                File.Delete(sLogFile);
            }


            tw.Flush();
            tw.Close();
        }

        private void backgroundWorkerRunAll_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            JobResult myResult = e.UserState as JobResult;
            UInt32 nTemp = 0;



            for (int i = 0; i < this.listView1.Items.Count; i++)
            {
                if (this.listView1.Items[i].SubItems[0].Text.Contains(myResult.sJob))
                {
                    this.listView1.Items[i].SubItems[1].Text = myResult.sStatus;
                    if (!myResult.fStarting)
                    {
                        int nPercent = (int)((this.nJobsCompleted / this.nTotalJobs) * 100);
                        this.progressBar1.Value = nPercent;

                        if (myResult.fPassed)
                        {
                            nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[2].Text) + 1;
                            this.listView1.Items[i].SubItems[2].Text = nTemp.ToString();
                        }
                        else
                        {
                            nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[3].Text) + 1;
                            this.listView1.Items[i].SubItems[3].Text = nTemp.ToString();
                        }
                        nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[4].Text) + myResult.nGoodDiscs;
                        this.listView1.Items[i].SubItems[4].Text = nTemp.ToString();
                        nTemp = Convert.ToUInt32(this.listView1.Items[i].SubItems[5].Text) + myResult.nBadDiscs;
                        this.listView1.Items[i].SubItems[5].Text = nTemp.ToString();
                    }
                }

            }
        }

        private void backgroundWorkerRunAll_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            MessageBox.Show("Testing Complete.  See Complete Report for details.\n\n" + this.sReportFile);

        }
    }
     public class JobResult
    {
        public string sJob;
        public string sStatus;
        public bool fPassed;
        public UInt32 nGoodDiscs;
        public UInt32 nBadDiscs;
         public bool fStarting;
        
        public JobResult()
        {
        }
    }
}