using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;

namespace SampleClient
{
    public enum DataFileType{DATA, AUDIO, IMAGE, NONE=99};
    public enum DataImageType { ISOLEVEL1, ISOLEVEL2, ISOLEVEL3, JOLIET, ISOLEVEL1_JOLIET, ISOLEVEL2_JOLIET, ISOLEVEL3_JOLIET, UDF, UDF201 };
    public enum ImageType { MODE_1_2048, MODE_2_2336, MODE_2_2352 };
    public enum PrintQualityValues {Low=0, Medium, Better, High, Best};

    public class DataEntry
    {
        public string File;
        public string FileTo;
        public DataFileType DataType;
        public UInt32 Session;
        public UInt32 PreGap;
        public bool SetPreGap;

        public DataEntry()
        {
            File = "";
            DataType = DataFileType.NONE;
            Session = 0;

        }
    }
    
    public class JobFile
    {
        public ArrayList Data;
        public DataImageType DataType;
        public ImageType ImgType;
        
        public UInt32 Importance;
        public UInt32 Copies;
        public UInt32 BurnSpeed;
        public PrintQualityValues PrintQuality;
        public UInt32 PrintID;
        public UInt32 PrintOM;
        public UInt32 DriveID;
        public UInt32 BinID;

        public string JobID;
        public string ClientID;
        public UInt32 DiscType;
        public string Robot;
        public string PrintLabel;
        public string VolumeName;
        public string MergeTitle;   //You can have more merge fields then this.  We are using 2 in the sample
        public string MergePicture;
        public string CheckFileOnDisc;
        public string CheckSystemIDOnDisc;
        public string CheckVolumeIDOnDisc;
        public string CheckPubIDOnDisc;
        public string CheckPreparerIDOnDisc;
        public string CheckAppIDOnDisc;
        public string PVDSystemID;
        public string PVDPublisherID;
        public string PVDPreparerID;
        public string PVDApplicationID;
        public string ReadDataTo;
        public string ReadDataFormat;
        public bool CreateSubFolders;

        public bool DeleteFiles;
        public bool CloseDisc;
        public bool Verify;
        public bool SetPrinterSettings;
        public bool ManLoadUnload;
        public bool UseAnyDrive;
        public bool TestRecord;
        public bool RejectIfNotBlank;
        public bool PreMasterData;
        public bool CheckDisc;
        public bool SetPVD;
        public bool DataSetNow;
        public bool DataMode2;
        public bool DataSAO;
        public bool PrintReject;
    


        public JobFile()
        {
            Data = new ArrayList();
            Robot = "";
            DataType = DataImageType.JOLIET;
  
            Importance = 4;
            Copies = 1;
            BurnSpeed = 0;
            PrintQuality = PrintQualityValues.Medium;
            PrintID = 200;
            PrintOM = 15;
            DriveID = 0;
            BinID = 0;
            DiscType = 0;
            PrintReject = false;

            JobID = "";
            ClientID = "";
            Robot = "";
            PrintLabel = "";
            VolumeName = "";
            ImgType = ImageType.MODE_1_2048;

            MergeTitle = "";   //You can have more merge fields then this.  We are using 2 in the sample
            MergePicture = "";
            CheckFileOnDisc = "";
            CheckSystemIDOnDisc = "";
            CheckVolumeIDOnDisc = "";
            CheckPubIDOnDisc = "";
            CheckPreparerIDOnDisc = "";
            CheckAppIDOnDisc = "";
            PVDSystemID = "";
            PVDPublisherID = "";
            PVDPreparerID = "";
            PVDApplicationID = "";

            DeleteFiles = false;
            CloseDisc = false;
            Verify = false;
            SetPrinterSettings = false;
            ManLoadUnload = false;
            UseAnyDrive = true;
            TestRecord = false;
            RejectIfNotBlank = false;
            PreMasterData = false;
            CheckDisc = false;
            SetPVD = false;
            DataSetNow = false;
            DataMode2 = false;
            DataSAO = false;
    
        }

        public bool LoadJobFile(string sFilePath)
        {
            string sLine;
            string sKey;
            TextReader tr = new StreamReader(sFilePath);
            while (true)
            {
                try
                {
                    sLine = tr.ReadLine();
                }
                catch
                {
                    return false;
                }
                sKey = "JobID=";
                if (!String.IsNullOrEmpty(sLine))
                {
                    if (sLine.StartsWith(sKey))
                        JobID = sLine.Replace(sKey, "");
                    sKey = "ClientID=";
                    if (sLine.StartsWith(sKey))
                        ClientID = sLine.Replace(sKey, "");
                    sKey = "Importance=";
                    if (sLine.StartsWith(sKey))
                        Importance = Convert.ToUInt32(sLine.Replace(sKey, ""));
                    sKey = "RobotName=";
                    if (sLine.StartsWith(sKey))
                        Robot = sLine.Replace(sKey, "");
                    sKey = "Copies=";
                    if (sLine.StartsWith(sKey))
                        Copies = Convert.ToUInt32(sLine.Replace(sKey, ""));
                }
                else
                    break;
            }
            return true;
        }
        public bool SendReadJob(string sFilePath)
        {
            bool fRet = false;
            if (!File.Exists(sFilePath))
            {
                TextWriter tw;
                if( Properties.Settings.Default.UnicodeEncoding)
                    tw = new StreamWriter(sFilePath, false, Encoding.Unicode);
                else
                    tw = new StreamWriter(sFilePath);

                if (tw != null)
                {
                    if (!String.IsNullOrEmpty(this.JobID))
                        tw.WriteLine("JobID=" + JobID);
                    if (!String.IsNullOrEmpty(this.ClientID))
                        tw.WriteLine("ClientID=" + ClientID);
                    
                    if (!String.IsNullOrEmpty(this.Robot))
                        tw.WriteLine("RobotName=" + Robot);
                    if (!UseAnyDrive)
                        tw.WriteLine("DriveID=" + this.DriveID.ToString());

                    tw.WriteLine("Importance=" + Importance.ToString());
                    if (this.BinID != 3)
                        tw.WriteLine("BinID=" + BinID.ToString());

                    tw.WriteLine("ReadDataTo=" + ReadDataTo);
                    tw.WriteLine("ReadDataFormat=" + ReadDataFormat);
                    if (CreateSubFolders)
                        tw.WriteLine("CreateSubFolders=YES");
                    else
                        tw.WriteLine("CreateSubFolders=NO");


                    tw.WriteLine("Copies=" + this.Copies.ToString());


                    if (this.ManLoadUnload)
                        tw.WriteLine("LoadUnloadOverride=YES");
                    else
                        tw.WriteLine("LoadUnloadOverride=NO");

                    tw.WriteLine("BurnSpeed=" + this.BurnSpeed.ToString());

                    if (this.CheckDisc)
                    {
                        if (!String.IsNullOrEmpty(this.CheckFileOnDisc))
                            tw.WriteLine("CheckFileOnDisc=" + this.CheckFileOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckSystemIDOnDisc))
                            tw.WriteLine("CheckSystemIDOnDisc=" + this.CheckSystemIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckPreparerIDOnDisc))
                            tw.WriteLine("CheckPreparerIDOnDisc=" + this.CheckPreparerIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckPubIDOnDisc))
                            tw.WriteLine("CheckPubIDOnDisc=" + this.CheckPubIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckAppIDOnDisc))
                            tw.WriteLine("CheckAppIDOnDisc=" + this.CheckAppIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckVolumeIDOnDisc))
                            tw.WriteLine("CheckVolumeIDOnDisc=" + this.CheckVolumeIDOnDisc);
                    }

                    tw.Close();
                    fRet = true;

                }
            }
            return fRet;
        }


        public bool SendJobFile(string sFilePath)
        {
            bool fRet = false;
            if (!File.Exists(sFilePath))
            {
                TextWriter tw;
                if( Properties.Settings.Default.UnicodeEncoding)
                    tw = new StreamWriter(sFilePath, false, Encoding.Unicode);
                else
                    tw = new StreamWriter(sFilePath);
                
                if (tw != null)
                {
                    if (!String.IsNullOrEmpty(this.JobID))
                        tw.WriteLine("JobID=" + JobID);
                    if (!String.IsNullOrEmpty(this.ClientID))
                        tw.WriteLine("ClientID=" + ClientID);
                    
                    if (!String.IsNullOrEmpty(this.Robot))
                        tw.WriteLine("RobotName=" + Robot);
                    if (!UseAnyDrive)
                        tw.WriteLine("DriveID=" + this.DriveID.ToString());

                    tw.WriteLine("Importance=" + Importance.ToString());
                    if (this.BinID != 3)
                        tw.WriteLine("BinID=" + BinID.ToString());

                    if (this.DiscType != 0)
                    {
                        if (this.DiscType == 1)
                            tw.WriteLine("DiscType=CDR");
                        else
                            tw.WriteLine("DiscType=DVDR");
                    }


                    bool fSetImageType = false;
                    bool fSetDataType = false;
                    foreach (DataEntry data in this.Data)
                    {
                        string sValue = "";
                        if (data.DataType == DataFileType.DATA)
                        {
                            sValue = "Data=" + data.File + ">" + data.FileTo;
                            if (!fSetDataType)
                            {
                                string DataTypeStr = "DataImageType=" + this.DataType.ToString().Replace('_', '+');
                                if (this.DataSetNow)
                                    DataTypeStr += ", SETNOW";
                                if (this.DataSAO)
                                    DataTypeStr += ", SAO";
                                if (this.DataMode2)
                                    DataTypeStr += ", MODE2";
                                tw.WriteLine(DataTypeStr);
                                fSetDataType = true;
                            }
                        }
                        else if (data.DataType == DataFileType.AUDIO)
                        {
                            sValue = "AudioFile=" + data.File;
                            if (data.SetPreGap)
                                sValue += " Pregap" + data.PreGap.ToString();
                        }
                        else if (data.DataType == DataFileType.IMAGE)
                        {
                            sValue = "ImageFile=" + data.File;
                            if (!fSetImageType && (Path.GetExtension(data.File).ToLower() == "iso"))
                            {
                                tw.WriteLine("ImageType="+this.ImgType.ToString());
                                fSetImageType = true;
                            }
                        }
                        if (data.Session != 0)
                            sValue += " Session" + data.Session.ToString();
                        tw.WriteLine(sValue);
                    }
                    if (!String.IsNullOrEmpty(this.PrintLabel))
                    {
                        tw.WriteLine("PrintLabel=" + this.PrintLabel);
                        if (this.PrintLabel.EndsWith("MergeExample.std"))
                        {
                            tw.WriteLine("MergeField=" + this.MergeTitle);
                            tw.WriteLine("MergeField=" + this.MergePicture);
                        }
                    }

                    tw.WriteLine("Copies=" + this.Copies.ToString());

                    if (this.SetPrinterSettings)
                    {
                        tw.WriteLine("PrintQuality=" + Convert.ToUInt32(this.PrintQuality));
                        tw.WriteLine("PrintInnerDiam=" + this.PrintID);
                        tw.WriteLine("PrintOuterMargin=" + this.PrintOM);
                    }
                    if (this.ManLoadUnload)
                        tw.WriteLine("LoadUnloadOverride=YES");
                    else
                        tw.WriteLine("LoadUnloadOverride=NO");

                    if (this.PrintReject)
                        tw.WriteLine("PrintReject=YES");
                    else
                        tw.WriteLine("PrintReject=NO");

                    if (this.TestRecord)
                        tw.WriteLine("TestRecord=YES");
                    else
                        tw.WriteLine("TestRecord=NO");

                    if (this.PreMasterData)
                        tw.WriteLine("PreMasterData=YES");
                    else
                        tw.WriteLine("PreMasterData=NO");

                    if (this.RejectIfNotBlank)
                        tw.WriteLine("RejectIfNotBlank=YES");
                    else
                        tw.WriteLine("RejectIfNotBlank=NO");

                    if (this.Verify)
                        tw.WriteLine("VerifyDisc=YES");
                    else
                        tw.WriteLine("VerifyDisc=NO");

                    if (this.CloseDisc)
                        tw.WriteLine("CloseDisc=YES");
                    else
                        tw.WriteLine("CloseDisc=NO");

                    if (this.DeleteFiles)
                        tw.WriteLine("DeleteFiles=YES");
                    else
                        tw.WriteLine("DeleteFiles=NO");

                    tw.WriteLine("BurnSpeed=" + this.BurnSpeed.ToString());
                    tw.WriteLine("VolumeName=" + this.VolumeName);

                    if (this.CheckDisc)
                    {
                        if (!String.IsNullOrEmpty(this.CheckFileOnDisc))
                            tw.WriteLine("CheckFileOnDisc=" + this.CheckFileOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckSystemIDOnDisc))
                            tw.WriteLine("CheckSystemIDOnDisc=" + this.CheckSystemIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckPreparerIDOnDisc))
                            tw.WriteLine("CheckPreparerIDOnDisc=" + this.CheckPreparerIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckPubIDOnDisc))
                            tw.WriteLine("CheckPubIDOnDisc=" + this.CheckPubIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckAppIDOnDisc))
                            tw.WriteLine("CheckAppIDOnDisc=" + this.CheckAppIDOnDisc);
                        if (!String.IsNullOrEmpty(this.CheckVolumeIDOnDisc))
                            tw.WriteLine("CheckVolumeIDOnDisc=" + this.CheckVolumeIDOnDisc);
                    }
                    if (this.SetPVD)
                    {
                        if (!String.IsNullOrEmpty(this.PVDSystemID))
                            tw.WriteLine("PVDSystemID=" + this.PVDSystemID);
                        if (!String.IsNullOrEmpty(this.PVDApplicationID))
                            tw.WriteLine("PVDApplicationID=" + this.PVDApplicationID);
                        if (!String.IsNullOrEmpty(this.PVDPreparerID))
                            tw.WriteLine("PVDPreparerID=" + this.PVDPreparerID);
                        if (!String.IsNullOrEmpty(this.PVDPublisherID))
                            tw.WriteLine("PVDPublisherID=" + this.PVDPublisherID);
                    }

                    tw.Close();
                    fRet = true;

                }
            }
            return fRet;
        }
    }

}
